<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:transform version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fr="http://mzlair.se/xslfractal.php">

	<!-- Setup output type -->
	<xsl:output method="xml" version="1.0"
		encoding="iso-8859-1" indent="yes"
		doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" 
		doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"/>

	<!-- Copy everything as default -->
	<xsl:template match="*">
		<xsl:copy>
			<xsl:copy-of select="@*"/>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>

	<!-- the fractal tag -->
	<xsl:template match="fr:fractal">
		<xsl:apply-templates mode="fractal">
			<xsl:with-param name="real" select="0"/>
			<xsl:with-param name="imaginary" select="0"/>
		</xsl:apply-templates>
	</xsl:template>

	<!-- Dont want any spaces in these -->
	<xsl:strip-space elements="fr:for-real fr:for-imaginary"/>

	<!-- loop over some real numbers -->
	<xsl:template match="fr:for-real" name="for-real" mode="fractal">
		<xsl:param name="imaginary"/>
		<xsl:param name="i" select="@steps"/>

		<xsl:if test="0 &lt; $i">
			<xsl:apply-templates mode="fractal">
				<xsl:with-param name="real" select="(@from - @to) * $i div @steps + @to"/>
				<xsl:with-param name="imaginary" select="$imaginary"/>
			</xsl:apply-templates>

			<xsl:call-template name="for-real">
				<xsl:with-param name="i" select="$i - 1"/>
				<xsl:with-param name="imaginary" select="$imaginary"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<!-- loop over some imaginary numbers -->
	<xsl:template match="fr:for-imaginary" name="for-imaginary" mode="fractal">
		<xsl:param name="real"/>
		<xsl:param name="i" select="@steps"/>

		<xsl:if test="0 &lt; $i">
			<xsl:apply-templates mode="fractal">
				<xsl:with-param name="real" select="$real"/>
				<xsl:with-param name="imaginary" select="(@from - @to) * $i div @steps + @to"/>
			</xsl:apply-templates>

			<xsl:call-template name="for-imaginary">
				<xsl:with-param name="i" select="$i - 1"/>
				<xsl:with-param name="real" select="$real"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<!-- iterating mandelbrot function -->
	<xsl:template match="fr:mandelbrot" name="mandelbrot-iterate" mode="fractal">
		<xsl:param name="real"/>
		<xsl:param name="imaginary"/>

		<!-- C value for mandelbrot X = X^2 + C -->
		<xsl:param name="C" select="$real"/>
		<xsl:param name="Ci" select="$imaginary"/>
		<xsl:param name="X" select="$C"/>
		<xsl:param name="Xi" select="$Ci"/>
		<xsl:param name="i" select="0"/>

		<!-- setup variables used -->
		<xsl:variable name="norm2" select="$X*$X + $Xi*$Xi"/>
		<xsl:variable name="norm-limit" select="16"/>
		<xsl:variable name="i-limit" select="@iteration-limit"/>

		<xsl:choose>
			<!-- should we iterate more? (|x|^2 < 4^2 and i < limit) -->
			<xsl:when test="$norm2 &lt; $norm-limit and $i &lt; $i-limit">
				<!-- iterate -->
				<xsl:call-template name="mandelbrot-iterate">
					<xsl:with-param name="real" select="$real"/>
					<xsl:with-param name="imaginary" select="$imaginary"/>

					<xsl:with-param name="X" select="$X*$X - $Xi*$Xi + $C"/>
					<xsl:with-param name="Xi" select="2*$X*$Xi + $Ci"/>
					<xsl:with-param name="i" select="1+$i"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$i=$i-limit">
				<!-- iteration limit found, output infinite "color" -->
				<xsl:apply-templates select="fr:infinite-color" mode="fractal"/>
			</xsl:when>
			<xsl:otherwise>
				<!-- outside of mandelbrot set, output iteration "color" -->
				<xsl:variable name="i-color" select="$i mod count(fr:color)+1"/>
				<xsl:apply-templates select="fr:color[position()=$i-color]" mode="fractal"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- iterating julia function -->
	<xsl:template match="fr:julia" name="julia-iterate" mode="fractal">
		<xsl:param name="real"/>
		<xsl:param name="imaginary"/>

		<!-- C value for mandelbrot X = X^2 + C -->
		<xsl:param name="C" select="@real"/>
		<xsl:param name="Ci" select="@imaginary"/>
		<xsl:param name="X" select="$real"/>
		<xsl:param name="Xi" select="$imaginary"/>
		<xsl:param name="i" select="0"/>

		<!-- setup variables used -->
		<xsl:variable name="norm2" select="$X*$X + $Xi*$Xi"/>
		<xsl:variable name="norm-limit" select="16"/>
		<xsl:variable name="i-limit" select="@iteration-limit"/>

		<xsl:choose>
			<xsl:when test="$norm2 &lt; $norm-limit and $i &lt; $i-limit">
				<xsl:call-template name="julia-iterate">
					<xsl:with-param name="real" select="$real"/>
					<xsl:with-param name="imaginary" select="$imaginary"/>

					<xsl:with-param name="X" select="$X*$X - $Xi*$Xi + $C"/>
					<xsl:with-param name="Xi" select="2*$X*$Xi + $Ci"/>
					<xsl:with-param name="i" select="1+$i"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$i=$i-limit">
				<xsl:apply-templates select="fr:infinite-color" mode="fractal"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="i-color" select="$i mod count(fr:color)+1"/>
				<xsl:apply-templates select="fr:color[position()=$i-color]" mode="fractal"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- dont include fr:infinite-color tag in output -->
	<xsl:template match="fr:infinite-color" mode="fractal">
		<xsl:apply-templates/>
	</xsl:template>

	<!-- dont include fr:infinite-color tag in output -->
	<xsl:template match="fr:color" mode="fractal">
		<xsl:apply-templates/>
	</xsl:template>

	<!-- Copy every tag inside fractal tags, while keeping track of real and imaginary numbers -->
	<xsl:template match="*" mode="fractal">
		<xsl:param name="real"/>
		<xsl:param name="imaginary"/>

		<xsl:copy>
			<xsl:copy-of select="@*"/>

			<xsl:apply-templates mode="fractal">
				<xsl:with-param name="real" select="$real"/>
				<xsl:with-param name="imaginary" select="$imaginary"/>
			</xsl:apply-templates>
		</xsl:copy>
	</xsl:template>

</xsl:transform>


